Reforgerade = LibStub("AceAddon-3.0"):NewAddon("Reforgerade", "AceConsole-3.0", "AceTimer-3.0")
local Reforgerade = Reforgerade
local reforgeString
local reforgeTable = {}
local RfInfo = LibStub("LibReforgingInfo-1.0")

local debugPrint = false
local debugErrors = true

local firstRun = true

local reforgeTooltips = {}

local ColorGreen = "|cff1eff00"

local reforgeradeOptionsTable = {
  type = "group",
  args = {
    debugEnable = {
      name = "Debug",
      desc = "Enables / disables debugging",
      type = "toggle",
      set = function(info,val) debugPrint = val end,
      get = function(info) return debugPrint end
    },
    debugErrors = {
      name = "Display Errors",
      desc = "Enables / disables displaying error messages",
      type = "toggle",
      set = function(info,val) debugErrors = val end,
      get = function(info) return debugErrors end
    },
  }
}
LibStub("AceConfig-3.0"):RegisterOptionsTable("Reforgerade", reforgeradeOptionsTable)
local optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Reforgerade")

Reforgerade:RegisterChatCommand("rfa", "ChatCommand")
Reforgerade:RegisterChatCommand("reforgerade", "ChatCommand")

function Reforgerade:ChatCommand(parameter)
	if not parameter or parameter:trim() == "" then
		ReforgeradeInputFrame.frame:Show()
	else
		parameter = parameter:trim():lower()
		if parameter == "options" or parameter == "config" then
			 InterfaceOptionsFrame_OpenToCategory(optionsFrame)
		end
	end	
end


--Stole from LibReforge
local SPI = 1
local DODGE = 2
local PARRY = 3
local HIT = 4
local CRIT = 5
local HASTE = 6
local EXP = 7
local MASTERY = 8

--Stole from LibReforge
local StatNames = {
	ITEM_MOD_SPIRIT_SHORT,
	ITEM_MOD_DODGE_RATING_SHORT,
	ITEM_MOD_PARRY_RATING_SHORT,
	ITEM_MOD_HIT_RATING_SHORT,
	ITEM_MOD_CRIT_RATING_SHORT,
	ITEM_MOD_HASTE_RATING_SHORT,
	ITEM_MOD_EXPERTISE_RATING_SHORT,
	ITEM_MOD_MASTERY_RATING_SHORT
}
StatNames[0] = "None"
local StatToString = {
	"ITEM_MOD_SPIRIT_SHORT",
	"ITEM_MOD_DODGE_RATING_SHORT",
	"ITEM_MOD_PARRY_RATING_SHORT",
	"ITEM_MOD_HIT_RATING_SHORT",
	"ITEM_MOD_CRIT_RATING_SHORT",
	"ITEM_MOD_HASTE_RATING_SHORT",
	"ITEM_MOD_EXPERTISE_RATING_SHORT",
	"ITEM_MOD_MASTERY_RATING_SHORT"
}


local reforgequeue = {}
local reforgeParsers = {}

function Reforgerade:DebugPrint(debugString)
	if (debugPrint) then
		self:Print("DEBUG: " .. debugString)
	end
end

function Reforgerade:ErrorPrint(errorString)
	if (debugErrors) then
		self:Print("|cffff0000 ERROR: " .. errorString .. "|r")
	end
end


--Checks to make sure an item has the stat to reforge from and does not have the stat to reforge to
--If false a reason is provided as to why the reforge failed validation
function Reforgerade:IsValidReforge(item, from, to, silent)
	silent = silent or false --default to false if called with nil
	if not item or not from or not to then return false end
	self:DebugPrint("Checking "  .. item .. " " .. from .. " -> " .. to .. " for validity")
	if (GetItemInfo(item)) then
		if (from <= 8 and to <= 8) then
			local statTable = {}
			local itemFound = false
			for slotID = 1, 18 do
				local itemLink = GetInventoryItemLink("player", slotID)
				if itemLink then
					if itemLink:match(item:gsub("%-", "%%%-")) then  --hyphens need to be escaped with '%'.  '%' also needs to be escaped
						GetItemStats(itemLink, statTable)
						itemFound = true
						break
					end
				end
			end
			
			if not itemFound then
				return false, "Unable to find item stats for " .. item .. ". check to make sure it is equipped."
			end
			
			if ((statTable[StatToString[from]] ~= nil and statTable[StatToString[from]] > 0) or from == 0) then
				if (statTable[StatToString[to]] == nil) then
					return true
				else
					if (not silent) then
						self:DebugPrint(item .. " already has " .. StatNames[to] .. ". " .. StatNames[from] .. " -> " .. StatNames[to] .. " is an invalid reforge option.")
					end
					return false, item .. " already has " .. StatNames[to] .. ". " .. StatNames[from] .. " -> " .. StatNames[to] .. " is an invalid reforge option."
				end
			else
				if (not silent) then
					self:DebugPrint(item .. " does not have " .. StatNames[from] .. ". " .. StatNames[from] .. " -> " .. StatNames[to] .. " is an invalid reforge option.")
				end
				return false, item .. " does not have " .. StatNames[from] .. ". " .. StatNames[from] .. " -> " .. StatNames[to] .. " is an invalid reforge option."
			end
		else
			if (not silent) then
				self:DebugPrint("Invalid reforge IDs for " .. item)
			end
			return false
		end
	else
		if (not silent) then
			self:DebugPrint(item .. " appears to be an invalid item.")
		end
		return false, item .. " appears to be an invalid item name."
	end
end

function Reforgerade:BuildReforgeTable(silent)
	silent = silent or false --default to false if called with nil
	local reforgeString = ReforgeradeInputFrame.editBox:GetText()
	reforgeTable = {}
	local best = 0
	local parserName = nil
	local pickedParser = nil
	if next(reforgeParsers) ~= nil then
		for parserName, parserFunction in pairs(reforgeParsers) do --for each parser added to our list let them try parsing the data
			local parseAttempt = parserFunction(reforgeString)
			local validItems = 0
			local attemptedReforges = 0
			if (parseAttempt and next(parseAttempt) ~= nil) then
				for itemName,reforgeID in pairs(parseAttempt) do
					if (Reforgerade:IsValidReforge(itemName, reforgeID[1], reforgeID[2], silent)) then
						validItems = validItems + 1					
					end
					attemptedReforges = attemptedReforges + 1
				end
			end
			if (not silent) then
				self:DebugPrint(parserName .. " returned " .. validItems .. " valid item reforges")
			end
			if ((validItems > best) or (best == 0 and attemptedReforges > 0)) then
				best = validItems
				reforgeTable = parseAttempt
				pickedParser = parserName
			end
		end
		
		if next(reforgeTable) ~= nil then
			if (not silent) then
				self:DebugPrint("Picked parser: " .. pickedParser)
			end
			local finalizedTable = {}
			for itemName,reforgeID in pairs(reforgeTable) do
				for i = 1, #reforgeID /2 do
					local duplicateIndex = i-1
					local duplicateOffset = duplicateIndex * 2
					local valid, reason = Reforgerade:IsValidReforge(itemName, reforgeID[1 + duplicateOffset], reforgeID[2 + duplicateOffset], silent)
					if (valid) then
						if not finalizedTable[itemName] then
							finalizedTable[itemName] = {}
						end
						finalizedTable[itemName][1 + duplicateOffset] = reforgeID[1 + duplicateOffset]
						finalizedTable[itemName][2 + duplicateOffset] = reforgeID[2 + duplicateOffset]
					else
						if (firstRun) then
							if (reason) then
								self:ErrorPrint(reason)
							else
								self:ErrorPrint("Removing invalid reforge request for " .. itemName)
							end
						end
					end
				end
			end
			reforgeTable = finalizedTable
		else
			if not silent then
				self:ErrorPrint("Unable to parse input")
			end
		end
--~ 		for k,v in pairs(reforgeTable) do
--~ 			print("Finalized Reforge " .. k .. " from " .. v[1] .. " to " .. v[2])
--~ 		end
	else
		self:ErrorPrint("No reforge parsers found!")
	end
end

function Reforgerade:ReforgeScan(silent)
	
	silent = silent or false --default to false if called with nil
--~ 	if ReforgeradeInputFrame.frame:IsShown() == false then ReforgeradeInputFrame.frame:Show() return end
	Reforgerade:BuildReforgeTable(silent) --rebuild the list of want reforges
	reforgequeue = {} --clear the reforge queue
	local duplicateIndexs = {} --used for detecting duplicate item names (should only be a problem with rings, trinkets, and weapons, but covers all cases.)
	local itemChanged = false
	Reforgerade:ResetEquipmentIcons()
	reforgeTooltips = { [ReforgeradeInputFrame.Icon[4]] = "This item slot can't be reforged.", [ReforgeradeInputFrame.Icon[19]] = "This item slot can't be reforged." }
	for i = 1, 18 do
--~ 	for i = 1, 2 do
		local item = GetInventoryItemLink("player", i)
		if (item and i~=4) then --if we have and item equiped in the slot and the slot is not the shirt or tabard slot
			local itemName = select(1,GetItemInfo(item))
			local WantReforgeFrom, WantReforgeTo
			if (reforgeTable[itemName] ~= nil) then
				duplicateOffset = duplicateIndexs[itemName] or 0
				WantReforgeFrom = reforgeTable[itemName][(1  + duplicateOffset *2)]
				WantReforgeTo = reforgeTable[itemName][(2 + duplicateOffset *2)]
--~ 				self:Print(itemName .. " duplicate number " .. duplicateOffset .. " " .. 1  + duplicateOffset *2 .. " " .. 2  + duplicateOffset *2)
--~ 				self:Print(WantReforgeFrom .. " " .. WantReforgeTo)
				if duplicateIndexs[itemName] then
					duplicateIndexs[itemName] = duplicateIndexs[itemName] + 1
				else
					duplicateIndexs[itemName] = 1
				end
				if WantReforgeFrom and WantReforgeTo then
					local CurrentReforgeFrom, CurrentReforgeTo
					if RfInfo:IsItemReforged(item) then
						CurrentReforgeFrom, CurrentReforgeTo = RfInfo:GetReforgedStatIDs(RfInfo:GetReforgeID(item))			
					end
					if (CurrentReforgeFrom == nil or CurrentReforgeTo == nil) then
						CurrentReforgeFrom = 0
						CurrentReforgeTo = 0
					end
			--~ 			self:Print(itemName .. " is reforged from " .. CurrentReforgeFrom .. " to " .. CurrentReforgeTo)
					if (CurrentReforgeFrom ~= WantReforgeFrom or CurrentReforgeTo ~= WantReforgeTo) then
						if (not silent) then
							self:Print(item .. " currently reforged from " .. StatNames[CurrentReforgeFrom] .. " -> " .. StatNames[CurrentReforgeTo] .. " needs to be reforged from " .. StatNames[WantReforgeFrom] .. " -> " .. StatNames[WantReforgeTo])
							itemChanged = true
						end
						tinsert(reforgequeue, {i, WantReforgeFrom, WantReforgeTo})
						reforgeTooltips[ReforgeradeInputFrame.Icon[i]] = "Currently reforged from " .. StatNames[CurrentReforgeFrom] .. " -> " .. StatNames[CurrentReforgeTo] .. "\nWanted reforged from " .. StatNames[WantReforgeFrom] .. " -> " .. StatNames[WantReforgeTo]
					else
						reforgeTooltips[ReforgeradeInputFrame.Icon[i]] = ColorGreen .. "Already reforged."
						if (not silent) then
	--~ 	 					self:Print("No need to change reforging for " ..item)
						end
					end
				end
			else
				ReforgeradeInputFrame.Icon[i]:SetBackdrop( { 
					bgFile = "Interface\\Icons\\INV_Misc_QuestionMark", 
					edgeFile = "Interface\\Addons\\Reforgerade\\media\\textures\\blank", tile = false, tileSize = 0, edgeSize = -1, 
					insets = { left = -2, right = -2, top = -2, bottom = -2 }
					});
				reforgeTooltips[ReforgeradeInputFrame.Icon[i]] = "No reforge info for item"
				if (not silent) then
	 				self:DebugPrint("Unable to find wanted reforge information for equiped item " .. item)
				end
			end
		end
	end
	if (not silent) then
		if (itemChanged == false) then
			self:Print("Everything is already reforged.")
		end
	end
	for k,v in pairs(reforgequeue) do
		ReforgeradeInputFrame.Icon[v[1]]:SetBackdrop( { 
					bgFile = GetInventoryItemTexture("player", v[1]), 
					edgeFile = "Interface\\Addons\\Reforgerade\\media\\textures\\blank", tile = false, tileSize = 0, edgeSize = -1, 
					insets = { left = -2, right = -2, top = -2, bottom = -2 }
					});
	end
end

local currentSlot = 0

function Reforgerade:ReforgeIt()
	Reforgerade:ReforgeScan(true)
--~ 	self:DebugPrint("Running ReforgingIt")
	if (#reforgequeue == 0 or reforgequeue == {} or reforgequeue == nil) then
		self:Print("Reforging Complete")
		firstRun = true
		if (ReforgingFrameItemButton and ReforgingFrameItemButton:IsVisible()) then
			ClearCursor()
			SetReforgeFromCursorItem() -- pick up the old item
			ClearCursor() --get rid of it
		end
		currentSlot = 0
		return
	end
	if (ReforgingFrame == nil  or ReforgingFrame:IsVisible() == nil) then
		self:ErrorPrint("Open the reforging frame")
		return
	end
	firstRun = false
	self:DebugPrint(#reforgequeue .. " items need to be reforged")
	for k, v in pairs(reforgequeue) do
--~ 		self:Print(v[1])
		--Pick up the item and place it in the reforge windo
		local itemSlot = v[1]
		local item = GetInventoryItemLink("player", itemSlot)
		if (currentSlot ~= itemSlot) then
			ClearCursor()
			SetReforgeFromCursorItem() -- pick up the old item
			ClearCursor() --get rid of it
			PickupInventoryItem(itemSlot)
			SetReforgeFromCursorItem()
			currentSlot = itemSlot
--~ 			self:DebugPrint("Placing " .. item .. " in the reforge window.")
		end
		local reforgeID
		if v[2] == 0 then
			reforgeID = 0
		else
			local stats = {GetSourceReforgeStats()};
			local fromStats = #stats/3
	--~ 		print("number of stats to reforge from " .. fromStats)
--~ 			local combinations = (8-fromStats)*fromStats
			local combinations = GetNumReforgeOptions()
	--~ 		print("total reforge combinations " .. combinations)
			for i=1, combinations do
				local from, _, _, to, _, _ = GetReforgeOptionInfo(i)
	--~ 			print(from .. " -> " .. to)
				if (StatNames[v[2]] == from and StatNames[v[3]] == to) then
					reforgeID = i
					break
				end
			end
		end
--~ 		self:DebugPrint("Found reforge combination reforge ID is " .. reforgeID)
		if reforgeID then
			self:DebugPrint("Reforging " .. item)
			ReforgeItem(reforgeID)
			Reforgerade:RetryReforge()
			return
		else
			self:ERRORPrint("Unable to find wanted reforge option for item " .. item)
		end
	end
	Reforgerade:RetryReforge()
end

function Reforgerade:RetryReforge()
	local waitTime = max(select(4, GetNetStats())*2, 250) -- Wait for 2x current latentcy or 250ms whichever is larger and do it again
--~ 	self:DebugPrint("Next action in " .. waitTime .. "ms")
	self:ScheduleTimer("ReforgeIt", waitTime/1000)  
end

function Reforgerade:RegisterParser(name, func)
	self:DebugPrint("Registering parser " .. name)
	reforgeParsers[name] = func
end



local SlotNames = {
   "HeadSlot",
   "NeckSlot",
   "ShoulderSlot",
   "ShirtSlot",
   "ChestSlot",
   "WaistSlot",
   "LegsSlot",
   "FeetSlot",
   "WristSlot",
   "HandsSlot",
   "Finger0Slot",
   "Finger1Slot",
   "Trinket0Slot",
   "Trinket1Slot",
   "BackSlot",
   "MainHandSlot",
   "SecondaryHandSlot",
   "RangedSlot",
   "TabardSlot",
}


--Stolen from shadowcraft-tools
ReforgeradeInputFrame = {}
ReforgeradeInputFrame.frame = nil
ReforgeradeInputFrame.frame = CreateFrame("Frame", "ReforgeradeInputFrame", UIParent)
ReforgeradeInputFrame.frame:SetBackdrop({
		bgFile = "Interface\\Addons\\Reforgerade\\media\\textures\\blank", --zomg stolen textures too
		edgeFile = "Interface\\Addons\\Reforgerade\\media\\textures\\blank",
		tile = true, tileSize = 16, edgeSize = 1, 
		insets = { left = -1, right = -1, top = -1, bottom = -1 }
})
ReforgeradeInputFrame.frame:RegisterForDrag("LeftButton")
ReforgeradeInputFrame.frame:SetMovable(true)
ReforgeradeInputFrame.frame:SetScript("OnDragStart", function(self) self:StartMoving() end)
ReforgeradeInputFrame.frame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() () end)
ReforgeradeInputFrame.frame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() () end)
ReforgeradeInputFrame.frame:SetScript("OnShow", function(self) ReforgeradeInputFrame.editBox:SetFocus() end)

ReforgeradeInputFrame.frame:SetBackdropColor(0.1,0.1,0.1,1)
ReforgeradeInputFrame.frame:SetBackdropBorderColor(0.6, 0.6, 0.6, 0.5)
ReforgeradeInputFrame.titleString = ReforgeradeInputFrame.frame:CreateFontString("TitleString");
ReforgeradeInputFrame.titleString:SetFont("Interface\\Addons\\Reforgerade\\media\\fonts\\arial.ttf",12)  --eek stolen fonts
ReforgeradeInputFrame.titleString:SetPoint("TOP", ReforgeradeInputFrame.frame, "TOP", -2, -2)
ReforgeradeInputFrame.titleString:SetText("Reforgerade")

ReforgeradeInputFrame.frame:SetWidth(400)
ReforgeradeInputFrame.frame:SetHeight(500)
ReforgeradeInputFrame.frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
ReforgeradeInputFrame.frame:Hide()
ReforgeradeInputFrame.frame:SetFrameStrata("DIALOG")

local scrollArea = CreateFrame("ScrollFrame", "ExportScroll", ReforgeradeInputFrame.frame, "UIPanelScrollFrameTemplate")

ReforgeradeInputFrame.editBox = CreateFrame("EditBox", "ReforgeradeEditBox", ReforgeradeInputFrame.frame)
ReforgeradeInputFrame.editBox:SetMultiLine(true)
ReforgeradeInputFrame.editBox:SetMaxLetters(99999)
ReforgeradeInputFrame.editBox:EnableMouse(true)
ReforgeradeInputFrame.editBox:SetAutoFocus(false)
ReforgeradeInputFrame.editBox:SetFontObject(ChatFontNormal)
ReforgeradeInputFrame.editBox:SetWidth(250)
ReforgeradeInputFrame.editBox:SetHeight(100)
ReforgeradeInputFrame.editBox:SetScript("OnEscapePressed", function(self) self:ClearFocus() end)
local lastEditTextLength = 0
ReforgeradeInputFrame.editBox:SetScript("OnTextChanged", function(self,isUserInput)
	if isUserInput then 
		if math.abs(#self:GetText() - lastEditTextLength) > 1 then --if more than 1 character was entered or deleted
			Reforgerade:ReforgeScan(true)
--~ 			print("pasted text")
		else
--~ 			print("typed text")  --or could be deleted text
		end
		lastEditTextLength = #self:GetText()
	end 
 end
 )
--~ ReforgeradeInputFrame.editBox:SetScript("OnMouseDown", function() ReforgeradeInputFrame.editBox:SetFocus() end)
--~ ReforgeradeInputFrame.frame:SetScript("OnMouseDown", function() ReforgeradeInputFrame.editBox:SetFocus() end)

scrollArea:SetPoint("TOPLEFT", ReforgeradeInputFrame.frame, "TOPLEFT", 55, -30)
scrollArea:SetPoint("BOTTOMRIGHT", ReforgeradeInputFrame.frame, "BOTTOMRIGHT", -75, 80)
scrollArea:SetScrollChild(ReforgeradeInputFrame.editBox)
scrollArea:SetScript("OnMouseDown", function(self,button) ReforgeradeInputFrame.editBox:SetFocus() end)

local close = CreateFrame("Button", "CopyCloseButton", ReforgeradeInputFrame.frame, "UIPanelCloseButton")
close:SetPoint("TOPRIGHT", ReforgeradeInputFrame.frame, "TOPRIGHT")

local ReforgeCompareButton = CreateFrame("Button", "ReforgeCompareButton", ReforgeradeInputFrame.frame, "OptionsButtonTemplate")
ReforgeCompareButton:SetPoint("BOTTOMRIGHT", ReforgeradeInputFrame.frame, "BOTTOM")
ReforgeCompareButton:SetText("Compare")
ReforgeCompareButton:SetScript("OnClick", function() Reforgerade:ReforgeScan(false) end)

local ReforgeItButton = CreateFrame("Button", "ReforgeItButton", ReforgeradeInputFrame.frame, "OptionsButtonTemplate")
ReforgeItButton:SetPoint("BOTTOMLEFT", ReforgeradeInputFrame.frame, "BOTTOM")
ReforgeItButton:SetText("Reforge It")
ReforgeItButton:SetScript("OnClick", function() firstRun = true currentSlot = 0 Reforgerade:ReforgeIt() end)

ReforgeradeInputFrame.frame:RegisterEvent("FORGE_MASTER_OPENED")
ReforgeradeInputFrame.frame:SetScript("OnEvent", function() ReforgeradeInputFrame.frame:Show() end)

ReforgeradeInputFrame.Tooltip = CreateFrame("GameTooltip", "ReforgeradeTooltip", UIParent, "GameTooltipTemplate")
ReforgeradeInputFrame.Tooltip:SetOwner(UIParent, "ANCHOR_CURSOR")


ReforgeradeInputFrame.Icon = {}
for i=1, 19 do
	ReforgeradeInputFrame.Icon[i] = CreateFrame("Frame", "ReforgeradeIcon" .. i, ReforgeradeInputFrame.frame)
	ReforgeradeInputFrame.Icon[i]:SetBackdrop( { 
					bgFile = select(2,GetInventorySlotInfo(SlotNames[i])), 
					edgeFile = "Interface\\Addons\\Reforgerade\\media\\textures\\blank", tile = false, tileSize = 0, edgeSize = -1, 
					insets = { left = -2, right = -2, top = -2, bottom = -2 }
					});
	ReforgeradeInputFrame.Icon[i]:SetWidth(40)
	ReforgeradeInputFrame.Icon[i]:SetHeight(40)
	ReforgeradeInputFrame.Icon[i]:SetScript("OnEnter", function(self)
		if reforgeTooltips[self] then 
			ReforgeradeInputFrame.Tooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
			ReforgeradeInputFrame.Tooltip:SetText(reforgeTooltips[self])
		else
			ReforgeradeInputFrame.Tooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
			ReforgeradeInputFrame.Tooltip:SetText("No reforge info for item")
		end
	end
	)
	ReforgeradeInputFrame.Icon[i]:SetScript("OnLeave", function(self) ReforgeradeInputFrame.Tooltip:Hide() end)
end

reforgeTooltips[ReforgeradeInputFrame.Icon[4]] = "This item slot can't be reforged."
reforgeTooltips[ReforgeradeInputFrame.Icon[19]] = "This item slot can't be reforged."

local IconPositions = {}
for i = 1, 19 do
	IconPositions[i] = {}
end
IconPositions[1] = { ofsx = 10, ofsy = -30, relativePoint = "TOPLEFT" }
IconPositions[2] = { relativeTo = ReforgeradeInputFrame.Icon[1] }
IconPositions[3] = { relativeTo = ReforgeradeInputFrame.Icon[2] }
IconPositions[15] = { relativeTo = ReforgeradeInputFrame.Icon[3] }
IconPositions[5] = { relativeTo = ReforgeradeInputFrame.Icon[15] }
IconPositions[4] = { relativeTo = ReforgeradeInputFrame.Icon[5] }
IconPositions[19] = { relativeTo = ReforgeradeInputFrame.Icon[4] }
IconPositions[9] = { relativeTo = ReforgeradeInputFrame.Icon[19] }
--Next Column
IconPositions[10] = { point = "TOPRIGHT", relativePoint = "TOPRIGHT", ofsx = -10, ofsy = -30}
IconPositions[6] = { relativeTo = ReforgeradeInputFrame.Icon[10] }
IconPositions[7] = { relativeTo = ReforgeradeInputFrame.Icon[6] }
IconPositions[8] = { relativeTo = ReforgeradeInputFrame.Icon[7] }
IconPositions[11] = { relativeTo = ReforgeradeInputFrame.Icon[8] }
IconPositions[12] = { relativeTo = ReforgeradeInputFrame.Icon[11] }
IconPositions[13] = { relativeTo = ReforgeradeInputFrame.Icon[12] }
IconPositions[14] = { relativeTo = ReforgeradeInputFrame.Icon[13] }
--weapons
IconPositions[16] = { point = "BOTTOMLEFT", relativePoint = "BOTTOMLEFT", ofsx = 132, ofsy = 30 }
IconPositions[17] = { relativeTo = ReforgeradeInputFrame.Icon[16], relativePoint = "TOPRIGHT", ofsx = 8, ofsy = 0 }
IconPositions[18] = { relativeTo = ReforgeradeInputFrame.Icon[17], relativePoint = "TOPRIGHT", ofsx = 8, ofsy = 0 }

for i=1, 19 do
--~ print(i, (IconPositions[i]["relativeTo"] or ReforgeradeInputFrame.frame):GetName())
ReforgeradeInputFrame.Icon[i]:SetPoint(
	(IconPositions[i]["point"] or "TOPLEFT"), 
	(IconPositions[i]["relativeTo"] or ReforgeradeInputFrame.frame),
	(IconPositions[i]["relativePoint"] or "BOTTOMLEFT"),
	(IconPositions[i]["ofsx"] or 0),
	(IconPositions[i]["ofsy"] or -8)
	)
end



function Reforgerade:ResetEquipmentIcons()
for i=1, 19 do
	ReforgeradeInputFrame.Icon[i]:SetBackdrop( { 
					bgFile = select(2,GetInventorySlotInfo(SlotNames[i])), 
					edgeFile = "Interface\\Addons\\Reforgerade\\media\\textures\\blank", tile = false, tileSize = 0, edgeSize = -1, 
					insets = { left = -2, right = -2, top = -2, bottom = -2 }
					});
	end
end
